@@ -42,11 +42,18 @@
 #include "filechooser.h"
 #include "mendeleiev.h"
 #include "mesomery.h"
+#include "mesomery-arrow.h"
 #include "reaction.h"
+#include "reactant.h"
+#include "reaction-step.h"
+#include "reaction-arrow.h"
 #include <bonobo/bonobo-ui-util.h>
 #include <sys/stat.h>
 #include <gconf/gconf-client.h>
-#include <gnome-vfs-module-2.0/libgnomevfs/gnome-vfs-mime.h>
+#include <libgnomevfs/gnome-vfs-mime.h>
+#include <libgnomevfs/gnome-vfs-ops.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
+#include <sstream>
 
 enum gcpFileTypes
 {
@@ -122,6 +129,7 @@ gcpApplication::gcpApplication ()
 	m_pActiveTool = NULL;
 	m_pNode = NULL;
 	FontName = g_strdup("Bitstream Vera Serif 12");
+	CurDir = g_get_current_dir ();
 	GConfClient* cli = gconf_client_get_default ();
 	if (cli) {
 		const char *value;
@@ -149,13 +157,13 @@ gcpApplication::gcpApplication ()
 
 	// search help file
 	string lang = getenv ("LANG");
-	HelpFilename = string (DATADIR"/gnome/help/") + string (PACKAGE"/") + lang, + "/"PACKAGE".xml";
+	HelpFilename = string (DATADIR"/gnome/help/") + string (PACKAGE"/") + lang + "/"PACKAGE".xml";
 	struct stat buf;
 	int err = stat(HelpFilename.c_str (), &buf);
 	if (err) {
 		if (lang.size () > 2) {
 			lang.erase (2, lang.size());
-			HelpFilename = string (DATADIR"/gnome/help/"PACKAGE"/") + lang, + "/"PACKAGE".xml";
+			HelpFilename = string (DATADIR"/gnome/help/"PACKAGE"/") + lang + "/"PACKAGE".xml";
 			err = stat(HelpFilename.c_str (), &buf);
 		}
 		if (err) {
@@ -360,7 +368,13 @@ bool gcpApplication::FileProcess(const gchar* filename, int filetype, bool bSave
 			else if (!strcmp(ext, ".pdb")) filetype = gcp_ft_PDB;
 			else if (!strcmp(ext, ".ent")) filetype = gcp_ft_PDB;
 			else if (!strcmp(ext, ".chm")) filetype = gcp_ft_ChemDraw;
-			else filetype = gcp_ft_Normal;
+			else {
+				GtkWidget* message = gtk_message_dialog_new (window, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, 
+															_("Sorry, format not supported!"));
+				gtk_dialog_run (GTK_DIALOG (message));
+				gtk_widget_destroy (message);
+				return true;
+			}
 		}
 		else filetype = gcp_ft_Normal;
 	}
@@ -386,7 +400,8 @@ bool gcpApplication::FileProcess(const gchar* filename, int filetype, bool bSave
 		}
 	gchar* filename2;
 	struct stat buf;
-	gint err;
+	bool err;
+	GnomeVFSURI *uri;
 	if (bSave)
 	{
 		if (ext)
@@ -398,9 +413,11 @@ bool gcpApplication::FileProcess(const gchar* filename, int filetype, bool bSave
 				filename2 = g_strdup_printf("%s%s", filename, ext);
 		}
 		else filename2 = g_strdup(filename);
-		err = stat(filename2, &buf);
+		uri = gnome_vfs_uri_new (filename2);
+		err = gnome_vfs_uri_exists (uri);
+		gnome_vfs_uri_unref (uri);
 		gint result = GTK_RESPONSE_YES;
-		if (!err)
+		if (err)
 		{
 			gchar * message = g_strdup_printf(_("File %s\nexists, overwrite?"), filename2);
 			GtkDialog* Box = GTK_DIALOG(gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, message));
@@ -425,7 +442,10 @@ bool gcpApplication::FileProcess(const gchar* filename, int filetype, bool bSave
 	}
 	else //loading
 	{
-		err = stat(filename, &buf);
+/*		uri = gnome_vfs_uri_new (filename);
+		err = !gnome_vfs_uri_exists (uri);
+		gnome_vfs_uri_unref (uri);*/
+		err = stat (filename, &buf);
 		if (err)
 		{
 			if (!ext)
@@ -459,7 +479,10 @@ bool gcpApplication::FileProcess(const gchar* filename, int filetype, bool bSave
 						filename2 = g_strdup(filename);
 				}
 			else filename2 = g_strdup_printf("%s%s",filename, ext);
-			if (stat(filename2, &buf))
+/*			uri = gnome_vfs_uri_new (filename2);
+			err = !gnome_vfs_uri_exists (uri);
+			gnome_vfs_uri_unref (uri);*/
+			if (stat (filename, &buf))
 			{
 				g_free(filename2);
 				filename2 = g_strdup(filename);
@@ -489,23 +512,48 @@ void gcpApplication::SaveWithBabel(const gchar* filename, const gchar* ext, gcpD
 void gcpApplication::OpenWithBabel(const gchar* filename, const gchar* ext, gcpDocument* pDoc, int Type)
 {
 	char *old_num_locale;
-	bool bNew = (pDoc == NULL);
+	bool bNew = (pDoc == NULL), local;
+	GnomeVFSFileInfo *info = NULL;
+	bool result;
 	try
 	{
 		if (!filename) throw (int) 0;
-		ifstream ifs;
-		ifs.open(filename);
-		if (!ifs) throw (int) 1;
-		old_num_locale = g_strdup(setlocale(LC_NUMERIC, NULL));
-		setlocale(LC_NUMERIC, "C");
-		OBMol Mol((io_type)Type);
-		OBFileFormat fileFormat;
-		fileFormat.ReadMolecule(ifs, Mol);
-		if (bNew) pDoc = new gcpDocument(this, true);
-		bool result = pDoc->ImportOB(Mol);
-		setlocale(LC_NUMERIC, old_num_locale);
-		g_free(old_num_locale);
-		ifs.close();
+		info = gnome_vfs_file_info_new ();
+		gnome_vfs_get_file_info (filename, info, GNOME_VFS_FILE_INFO_DEFAULT);
+		local = GNOME_VFS_FILE_INFO_LOCAL (info);
+		gnome_vfs_file_info_unref (info);
+		if (bNew)
+			pDoc = new gcpDocument (this, true);
+		if (local) {
+			ifstream ifs;
+			ifs.open (filename);
+			if (!ifs) throw (int) 1;
+			old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
+			setlocale(LC_NUMERIC, "C");
+			OBMol Mol ((io_type) Type);
+			OBFileFormat fileFormat;
+			fileFormat.ReadMolecule (ifs, Mol);
+			result = pDoc->ImportOB(Mol);
+			setlocale (LC_NUMERIC, old_num_locale);
+			g_free (old_num_locale);
+			ifs.close ();
+		} else {
+			char *buf;
+			int size;
+			if (gnome_vfs_read_entire_file (filename, &size, &buf) != GNOME_VFS_OK)
+				throw 1;
+			istringstream iss (buf);
+			old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
+			setlocale(LC_NUMERIC, "C");
+			OBMol Mol ((io_type) Type);
+			OBFileFormat fileFormat;
+			fileFormat.ReadMolecule (iss, Mol);
+			result = pDoc->ImportOB(Mol);
+			setlocale (LC_NUMERIC, old_num_locale);
+			setlocale (LC_NUMERIC, old_num_locale);
+			g_free (old_num_locale);
+			g_free (buf);
+		}
 		if (!result)
 		{
 			if (bNew) delete pDoc;
@@ -544,14 +592,39 @@ void gcpApplication::SaveGcp(const gchar* filename, gcpDocument* pDoc)
 	pDoc->Save();
 }
 
+static int	cb_vfs_to_xml (GnomeVFSHandle *handle, char* buf, int nb)
+{
+	GnomeVFSFileSize ndone;
+	return  (gnome_vfs_read (handle, buf, nb, &ndone) == GNOME_VFS_OK)? (int) ndone: -1;
+}
+
 void gcpApplication::OpenGcp(const gchar* filename, gcpDocument* pDoc)
 {
 	xmlDocPtr xml;
 	char *old_num_locale, *old_time_locale;
+	GnomeVFSFileInfo *info = NULL;
 	try
 	{
-		if (!filename) throw (int) 0;
-		if (!(xml = xmlParseFile(filename))) throw (int) 1;
+		if (!filename)
+			throw (int) 0;
+
+		// try opening with write access to see if it is readonly
+		// use xmlReadIO for non local files.
+		info = gnome_vfs_file_info_new ();
+		gnome_vfs_get_file_info (filename, info, GNOME_VFS_FILE_INFO_DEFAULT);
+
+		if (GNOME_VFS_FILE_INFO_LOCAL (info)) {
+			if (!(xml = xmlParseFile(filename)))
+				throw (int) 1;
+		} else {
+			GnomeVFSHandle *handle;
+			GnomeVFSResult result = gnome_vfs_open (&handle, filename, GNOME_VFS_OPEN_READ);
+			if (result != GNOME_VFS_OK)
+				throw 1;
+			if (!(xml = xmlReadIO ((xmlInputReadCallback) cb_vfs_to_xml, 
+					 (xmlInputCloseCallback) gnome_vfs_close, handle, filename, NULL, 0)))
+				throw 1;
+		}
 		if (xml->children == NULL) throw (int) 2;
 		if (strcmp((char*)xml->children->name, "chemistry")) throw (int) 3;	//FIXME: that could change when a dtd is available
 		old_num_locale = g_strdup(setlocale(LC_NUMERIC, NULL));
@@ -571,10 +644,15 @@ void gcpApplication::OpenGcp(const gchar* filename, gcpDocument* pDoc)
 			throw (int) 4;
 		}
 		AddDocument(pDoc);
+		if (!(info->permissions & (GNOME_VFS_PERM_USER_WRITE | GNOME_VFS_PERM_GROUP_WRITE)))
+			pDoc->SetReadOnly (true);
+		gnome_vfs_file_info_unref (info);
 		xmlFreeDoc(xml);
 	}
 	catch (int num)
 	{
+		if (info)
+			gnome_vfs_file_info_unref (info);
 		if (num > 1) xmlFreeDoc(xml);
 		gchar *mess;
 		GtkWidget* message;
@@ -744,7 +822,7 @@ void gcpApplication::OnSaveAsImage ()
 		}
 		else {
 			GtkWidget* message = gtk_message_dialog_new (GTK_WINDOW (dialog), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, 
-																_("Sorry, format not supported !"));
+																_("Sorry, format not supported!"));
 			gtk_dialog_run (GTK_DIALOG (message));
 			gtk_widget_destroy (message);
 			continue;
@@ -754,3 +832,10 @@ void gcpApplication::OnSaveAsImage ()
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 	return;	
 }
+
+void gcpApplication::SetCurDir (char const* dir)
+{
+	if (CurDir)
+		g_free (CurDir);
+	CurDir = g_strdup (dir);
+}
