@@ -4,7 +4,7 @@
  * GChemPaint library
  * document.cc 
  *
- * Copyright (C) 2001-2004
+ * Copyright (C) 2001-2005
  *
  * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
@@ -44,8 +44,12 @@
 #include "text.h"
 #include <locale.h>
 #include <unistd.h>
+#include <libgen.h>
 #include <gtk/gtk.h>
+#include <libgnomevfs/gnome-vfs-file-info.h>
+#include <libgnomevfs/gnome-vfs-ops.h>
 #include <iostream>
+#include <sstream>
 
 using namespace OpenBabel;
 using namespace std;
@@ -75,6 +79,7 @@ gcpDocument::gcpDocument(gcpApplication *App, bool StandAlone): Document()
 	else m_mail = NULL;
 	m_comment = NULL;
 	m_pCurOp = NULL;
+	m_bReadOnly = false;
 	SetActive();
 }
 
@@ -111,43 +116,28 @@ const gchar* gcpDocument::GetLabel()
 	return m_label;
 }
 
-void gcpDocument::SetFileName(const gchar* Name, const gchar* ext)
+void gcpDocument::SetFileName (const gchar* Name, const gchar* ext)
 {
-	if (m_filename) g_free(m_filename);
-	if (*Name == '/') m_filename = g_strdup(Name);
-	else
-	{
-		gchar* curdir = g_get_current_dir();
-		m_filename = g_strconcat(curdir, "/", Name, NULL);
-		g_free(curdir);
-	}
-	int i = strlen(m_filename) - 1;
-	while ((m_filename[i] != '/') && (i >= 0)) i--;
-	if (i >=0) 
-	{
-		m_filename[i] = 0;
-		chdir(m_filename);
-		m_filename[i] = '/';
-	}
+	if (m_filename) g_free (m_filename);
+	m_filename = g_strdup (Name);
+	m_pApp->SetCurDir (dirname (m_filename));
+	g_free (m_filename);
+	m_filename = g_strdup (Name);
+	int i = strlen (m_filename) - 1;
+	while ((m_filename[i] != '/') && (i >= 0))
+		i--;
 	i++;
-	int j = strlen(m_filename) - 1;
-	while ((i < j) && (m_filename[j] != '.')) j--;
+	int j = strlen (m_filename) - 1;
+	while ((i < j) && (m_filename[j] != '.'))
+		j--;
 	if (m_label) g_free(m_label);
-	if (strcmp(m_filename + j, ext)) m_label = g_strdup(m_filename + i);
-	else m_label = g_strndup(m_filename + i, j - i);
-	if (m_pView) m_pView->UpdateLabel(m_label);
+	if (strcmp(m_filename + j, ext))
+		m_label = g_strdup (m_filename + i);
+	else
+		m_label = g_strndup (m_filename + i, j - i);
+	if (m_pView) m_pView->UpdateLabel (m_label);
 }
 
-/*void gcpDocument::FillCanvas()
-{
-	AnalTree();
-	std::map<std::string, gcpBond*>::iterator j;
-	for (j = m_Bonds.begin(); j != m_Bonds.end(); j++) m_pView->AddBond((*j).second);
-/*	std::map<std::string, gcpAtom*>::iterator i;
-	for (i = m_Atoms.begin(); i != m_Atoms.end(); i++) m_pView->AddAtom((*i).second);
-}*/
-
-
 void gcpDocument::BuildBondList(list<gcpBond*>& BondList, Object* obj)
 {
 	Object* pObject;
@@ -299,19 +289,26 @@ void gcpDocument::ExportOB()
 	Mol.EndModify();
 	try
 	{
-		ofstream ofs;
-		ofs.open(m_filename);
-		if (!ofs) throw (int) 1;
+		ostringstream ofs;
+		GnomeVFSHandle *handle = NULL;
+		GnomeVFSFileSize n;
+		GnomeVFSResult res;
+		if ((res = gnome_vfs_create(&handle, m_filename, GNOME_VFS_OPEN_WRITE, true, 0644)) != GNOME_VFS_OK)
+			throw (int) res;
 		old_num_locale = g_strdup(setlocale(LC_NUMERIC, NULL));
 		setlocale(LC_NUMERIC, "C");
 		OBFileFormat fileFormat;
 		fileFormat.WriteMolecule(ofs, Mol, "2D");
 		setlocale(LC_NUMERIC, old_num_locale);
 		g_free(old_num_locale);
-		ofs.close();
+		if ((res = gnome_vfs_write (handle, ofs.str ().c_str (), (GnomeVFSFileSize) ofs.str ().size (), &n)) != GNOME_VFS_OK)
+			throw (int) res;
+		gnome_vfs_close (handle);
+		SetReadOnly (false);
 	}
 	catch (int n)
 	{
+		printf("error #%d\n",n);
 	}
 }
 
@@ -336,12 +333,10 @@ void gcpDocument::AddAtom(gcpAtom* pAtom)
 		pAtom->SetId(id);
 		Id = id;
 	}
+	m_pView->AddObject(pAtom);
 	if (m_bIsLoading)
-	{
-		m_pView->AddObject(pAtom);
 		return;
-	}
-	AddObject(pAtom);
+//	AddObject(pAtom);
 	int i0 = 1;
 	gcpMolecule* mol = new gcpMolecule();
 	i = 1;
@@ -371,12 +366,10 @@ void gcpDocument::AddFragment(gcpFragment* pFragment)
 		pFragment->SetId(id);
 		Id = id;
 	}
+	m_pView->AddObject(pFragment);
 	if (m_bIsLoading)
-	{
-		m_pView->AddObject(pFragment);
 		return;
-	}
-	AddObject(pFragment);
+//	AddObject(pFragment);
 	pFragment->AnalContent();
 	if (!pFragment->GetMolecule())
 	{
@@ -412,12 +405,10 @@ void gcpDocument::AddBond(gcpBond* pBond)
 	gcpAtom *pAtom0 = (gcpAtom*)pBond->GetAtom(0), *pAtom1 = (gcpAtom*)pBond->GetAtom(1);
 	m_pView->Update(pAtom0);
 	m_pView->Update(pAtom1);
+	m_pView->AddObject(pBond);
 	if (m_bIsLoading)
-	{
-		m_pView->AddObject(pBond);
 		return;
-	}
-	AddObject(pBond);
+//	AddObject(pBond);
 	//search molecules
 	gcpMolecule * pMol0 = (gcpMolecule*)pAtom0->GetMolecule(),  *pMol1 = (gcpMolecule*)pAtom1->GetMolecule();
 	if (pMol0 && pMol1)
@@ -455,7 +446,7 @@ void gcpDocument::AddBond(gcpBond* pBond)
 
 void gcpDocument::Save()
 {
-	if (!m_filename || !m_bWriteable) return;
+	if (!m_filename || !m_bWriteable || m_bReadOnly) return;
 	if (m_FileType != UNDEFINED)
 	{
 		ExportOB();
@@ -564,6 +555,12 @@ void gcpDocument::ParseXMLTree (xmlDocPtr xml)
 	Load (xml->children);
 }
 
+static int cb_xml_to_vfs (GnomeVFSHandle *handle, const char* buf, int nb)
+{
+	GnomeVFSFileSize ndone;
+	return (int) gnome_vfs_write (handle, buf, nb, &ndone);
+}
+
 xmlDocPtr gcpDocument::BuildXMLTree()
 {
 	xmlDocPtr xml;
@@ -623,7 +620,30 @@ xmlDocPtr gcpDocument::BuildXMLTree()
 
 		xmlIndentTreeOutput = true;
 		xmlKeepBlanksDefault(0);
-		if (xmlSaveFormatFile(m_filename, xml, true) < 0) /*Error(SAVE)*/;
+
+		GnomeVFSFileInfo *info = gnome_vfs_file_info_new ();
+		gnome_vfs_get_file_info (m_filename, info, GNOME_VFS_FILE_INFO_DEFAULT);
+
+		if (GNOME_VFS_FILE_INFO_LOCAL (info)) {
+			gnome_vfs_file_info_unref (info);
+			if (xmlSaveFormatFile (m_filename, xml, true) < 0) /*Error(SAVE)*/;
+		} else {
+			gnome_vfs_file_info_unref (info);
+			xmlOutputBufferPtr buf = xmlAllocOutputBuffer (NULL);
+			GnomeVFSHandle *handle;
+			GnomeVFSResult result = gnome_vfs_open (&handle, m_filename, GNOME_VFS_OPEN_WRITE);
+			if (result == GNOME_VFS_ERROR_NOT_FOUND)
+				result = gnome_vfs_create (&handle, m_filename, GNOME_VFS_OPEN_WRITE, true, 0666);
+			if (result != GNOME_VFS_OK)
+				throw 1;
+			buf->context = handle;
+			buf->closecallback = (xmlOutputCloseCallback) gnome_vfs_close;
+			buf->writecallback = (xmlOutputWriteCallback) cb_xml_to_vfs;
+			int n = xmlSaveFormatFileTo (buf, xml, NULL, true);
+			if (n < 0)
+				throw 1;
+			SetReadOnly (false);
+		}
 //FIXME: the xml is freed and then we return it!!!
 		xmlFreeDoc(xml);
 		m_bDirty = false;
@@ -694,59 +714,76 @@ void gcpDocument::RemoveFragment(gcpFragment* pFragment)
 	delete pFragment;
 }
 
-void gcpDocument::RemoveBond(gcpBond* pBond)
+void gcpDocument::RemoveBond (gcpBond* pBond)
 {
-	m_pView->Remove(pBond);
+	m_pView->Remove (pBond);
 	gcpAtom *pAtom0, *pAtom1;
-	pAtom0 = (gcpAtom*)pBond->GetAtom(0);
-	pAtom1 = (gcpAtom*)pBond->GetAtom(1);
-	gcpMolecule *pMol = (gcpMolecule*)pBond->GetMolecule();
+	pAtom0 = (gcpAtom*) pBond->GetAtom (0);
+	pAtom1 = (gcpAtom*) pBond->GetAtom (1);
+	gcpMolecule *pMol = (gcpMolecule*) pBond->GetMolecule ();
 	char id[16];
-	pAtom0->RemoveBond(pBond);
-	m_pView->Update(pAtom0);
-	pAtom1->RemoveBond(pBond);
-	m_pView->Update(pAtom1);
-	if (pBond->IsCyclic())
-	{
-		pMol->Remove(pBond);
-		pMol->UpdateCycles();
-		Update();
-	}
-	else
-	{
+	pAtom0->RemoveBond (pBond);
+	m_pView->Update (pAtom0);
+	pAtom1->RemoveBond (pBond);
+	m_pView->Update (pAtom1);
+	if (pBond->IsCyclic ()) {
+		pMol->Remove (pBond);
+		pMol->UpdateCycles ();
+		Update ();
+	} else {
+		Object *Parent = pMol->GetParent ();
 		int i0 = 1;
+		string align_id = pMol->GetAlignmentId ();
 		delete pMol;
-		gcpMolecule * pMol = new gcpMolecule();
-		AddChild(pMol);
-		do snprintf(id, sizeof(id), "m%d", i0++);
-		while (GetDescendant(id) != NULL);
-		pMol->SetId(id);
-		Object* pObject = pAtom0->GetParent();
-		if (pObject->GetType() == FragmentType) pMol->AddFragment((gcpFragment*)pObject);
-		else pMol->AddAtom(pAtom0);
-		pMol->UpdateCycles();
-		do snprintf(id, sizeof(id), "m%d", i0++);
-		while (GetDescendant(id) != NULL);
-		pMol = new gcpMolecule();
-		pMol->SetId(id);
-		AddChild(pMol);
-		pObject = pAtom1->GetParent();
-		if (pObject->GetType() == FragmentType) pMol->AddFragment((gcpFragment*)pObject);
-		else pMol->AddAtom(pAtom1);
-		pMol->UpdateCycles();
-		if ((pAtom0->GetZ() == 6) && (pAtom0->GetBondsNumber() == 0)) m_pView->Update(pAtom0);
-		if ((pAtom1->GetZ() == 6) && (pAtom1->GetBondsNumber() == 0)) m_pView->Update(pAtom1);
+		gcpMolecule * pMol = new gcpMolecule ();
+		pMol->Lock (true);
+		do
+			snprintf (id, sizeof (id), "m%d", i0++);
+		while (GetDescendant (id) != NULL);
+		pMol->SetId (id);
+		Parent->AddChild (pMol); /* do not update at this point it's the caller responsibility */
+		Object* pObject = pAtom0->GetParent ();
+		if (pObject->GetType () == FragmentType)
+			pMol->AddFragment ((gcpFragment*) pObject);
+		else
+			pMol->AddAtom (pAtom0);
+		pMol->UpdateCycles ();
+		if (align_id.size ()) {
+			Object *obj = pMol->GetDescendant (align_id.c_str ());
+			if (obj)
+				pMol->SelectAlignmentItem (obj);
+			align_id = "";
+		}
+		pMol->Lock (false);
+		do
+			snprintf (id, sizeof (id), "m%d", i0++);
+		while (GetDescendant (id) != NULL);
+		pMol = new gcpMolecule ();
+		pMol->Lock (true);
+		pMol->SetId (id);
+		Parent->AddChild (pMol); /* do not update at this point it's the caller responsibility */
+		pObject = pAtom1->GetParent ();
+		if (pObject->GetType () == FragmentType)
+			pMol->AddFragment ((gcpFragment*) pObject);
+		else
+			pMol->AddAtom (pAtom1);
+		pMol->UpdateCycles ();
+		if (align_id.size ()) {
+			Object *obj = pMol->GetDescendant (align_id.c_str ());
+			if (obj)
+				pMol->SelectAlignmentItem (obj);
+		}
+		pMol->Lock (false);
+		if ((pAtom0->GetZ () == 6) && (pAtom0->GetBondsNumber () == 0))
+			m_pView->Update (pAtom0);
+		if ((pAtom1->GetZ () == 6) && (pAtom1->GetBondsNumber () == 0))
+			m_pView->Update (pAtom1);
 	}
 	delete pBond;
 }
 	
 void gcpDocument::Remove(Object* pObject)
 {
-	if (!m_bUndoRedo)
-	{
-		if (!m_pCurOp) m_pCurOp = new gcpDeleteOperation(this);
-		m_pCurOp->AddObject(pObject);
-	}
 	switch(pObject->GetType())
 	{
 		case AtomType:
@@ -831,8 +868,10 @@ void gcpDocument::AddObject(Object* pObject)
 	if (!pObject->GetParent()) AddChild(pObject);
 	m_pView->AddObject(pObject);
 	if (m_bIsLoading || m_bUndoRedo) return;
-	if (!m_pCurOp) m_pCurOp = new gcpAddOperation(this);
-	m_pCurOp->AddObject(pObject);
+	if (!m_pCurOp) {
+		m_pCurOp = new gcpAddOperation(this);
+		m_pCurOp->AddObject(pObject);
+	}
 }
 
 void gcpDocument::OnUndo()
@@ -929,6 +968,8 @@ void gcpDocument::SetActive ()
 	m_pApp->ActivateMenu("Undo", !m_UndoList.empty ());
 	m_pApp->ActivateMenu("Redo", !m_RedoList.empty ());
 	m_pApp->ActivateMenu("Image", HasChildren ());
+	m_pApp->ActivateMenu ("Save", !m_bReadOnly);
+	m_pApp->ActivateToolItem ("Save", !m_bReadOnly);
 }
 
 extern xmlDocPtr pXmlDoc;
@@ -1025,3 +1066,12 @@ void gcpDocument::ExportImage (const char* filename, const char* type)
 {
 	m_pView->ExportImage (filename, type);
 }
+
+void gcpDocument::SetReadOnly (bool ro)
+{
+	m_bReadOnly = ro;
+	if (m_pApp->GetActiveDocument () == this) {
+		m_pApp->ActivateMenu ("Save", !ro);
+		m_pApp->ActivateToolItem ("Save", !ro);
+	}
+}
