@@ -4,9 +4,9 @@
  * GChemPaint library
  * view.cc
  *
- * Copyright (C) 2001-2004
+ * Copyright (C) 2001-2005
  *
- * Developed by Jean Bréfort <jean.brefort@ac-dijon.fr>
+ * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -497,29 +497,57 @@ void gcpView::UpdateLabel(const gchar* label)
 	if (GTK_IS_LABEL(l->data)) gtk_label_set_text((GtkLabel*)(l->data), label);
 }
 
-void gcpView::OnDeleteSelection(GtkWidget* w)
+void gcpView::OnDeleteSelection (GtkWidget* w)
 {
 	m_pWidget = w;
-	gcpApplication*	pApp = m_pDoc->GetApplication();
-	gcpTool* pActiveTool = pApp->GetActiveTool();
-	if (!pActiveTool->DeleteSelection())
+	gcpApplication*	pApp = m_pDoc->GetApplication ();
+	gcpTool* pActiveTool = pApp->GetActiveTool ();
+	if (!pActiveTool->DeleteSelection ())
 	{
-		m_pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(w), "data");
+		m_pData = (gcpWidgetData*) g_object_get_data (G_OBJECT(w), "data");
 		gcpWidgetData* pData;
 		std::list<GtkWidget*>::iterator j;
-		for (j = m_Widgets.begin(); j != m_Widgets.end(); j++)
-		{
-			if (*j == m_pWidget) continue;
-			pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(*j), "data");
-			pData->UnselectAll();
+		for (j = m_Widgets.begin (); j != m_Widgets.end (); j++) {
+			if (*j == m_pWidget)
+				continue;
+			pData = (gcpWidgetData*) g_object_get_data(G_OBJECT(*j), "data");
+			pData->UnselectAll ();
+		}
+		Object *pObject, *Group;
+		set<string> ModifiedObjects;
+		bool modify = false;
+		list<Object *>::iterator i, iend = m_pData->SelectedObjects.end ();
+		// first search if we are deleting top objects or not
+		for (i = m_pData->SelectedObjects.begin (); i != iend; i++)
+			if ((*i)->GetGroup ()) {
+				modify = true;
+				break;
+			}
+		gcpOperation *Op = m_pDoc->GetNewOperation (((modify)?
+								GCP_MODIFY_OPERATION: GCP_DELETE_OPERATION));
+		while (!m_pData->SelectedObjects.empty ()) {
+			pObject = m_pData->SelectedObjects.front ();
+			Group = pObject->GetGroup ();
+			if (Group && 
+				(ModifiedObjects.find (Group->GetId ()) == ModifiedObjects.end ())) {
+				Op->AddObject (Group);
+				ModifiedObjects.insert (Group->GetId ());
+			} else
+				Op->AddObject (pObject);
+			m_pDoc->Remove (m_pData->SelectedObjects.front ());
+		}
+		m_pData->ClearSelection ();
+		set<string>::iterator k, kend = ModifiedObjects.end();
+		for (k = ModifiedObjects.begin(); k != kend; k++) {
+			pObject = m_pDoc->GetDescendant ((*k).c_str ());
+			if (pObject)
+				Op->AddObject (pObject, 1);
 		}
-		while (!m_pData->SelectedObjects.empty()) m_pDoc->Remove(m_pData->SelectedObjects.front());
-		m_pData->ClearSelection();
 	}
-	m_pDoc->FinishOperation();
-	pApp->ActivateMenu("Copy", false);
-	pApp->ActivateMenu("Cut", false);
-	pApp->ActivateMenu("Erase", false);
+	m_pDoc->FinishOperation ();
+	pApp->ActivateMenu ("Copy", false);
+	pApp->ActivateMenu ("Cut", false);
+	pApp->ActivateMenu ("Erase", false);
 }
 
 GtkTargetEntry const targets[] = {
