@@ -4,9 +4,9 @@
  * GChemPaint cycles plugin
  * cycletool.cc 
  *
- * Copyright (C) 2001-2004
+ * Copyright (C) 2001-2005
  *
- * Developed by Jean Bréfort <jean.brefort@ac-dijon.fr>
+ * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -193,6 +193,8 @@ bool gcpCycleTool::OnClicked()
 				switch (i)
 				{
 					case 0:
+						m_x0 = x1;
+						m_y0 = y1;
 						 bDone = false;
 						break;
 					case 1:
@@ -525,25 +527,30 @@ void gcpCycleTool::OnDrag()
 
 void gcpCycleTool::OnRelease()
 {
-	if (!m_size) return;
+	if (!m_size)
+		return;
 	double x1, y1, x2, y2;
-	if (m_Chain)
-	{
+	if (m_Chain) {
 		delete m_Chain;
 		m_Chain = NULL;
 	}
-	if (m_pItem)
-	{
-		gtk_object_destroy(GTK_OBJECT(GNOME_CANVAS_ITEM(m_pItem)));
+	if (m_pItem) {
+		gtk_object_destroy (GTK_OBJECT (GNOME_CANVAS_ITEM (m_pItem)));
 		m_pItem = NULL;
 	}
-	else return;
-	if (!m_bAllowed) return;
-	m_pApp->ClearStatus();
+	else
+		return;
+	if (!m_bAllowed)
+		return;
+	m_pApp->ClearStatus ();
 	gcpAtom* pAtom[m_size];
 	gcpBond* pBond;
+	Object *pObject;
+	char const *Id;
 	GnomeCanvasItem* pItem;
-	gcpDocument* pDoc = m_pView->GetDoc();
+	gcpDocument *pDoc = m_pView->GetDoc ();
+	gcpOperation *pOp = NULL;
+	gcpMolecule *pMol = NULL;
 	int i;
 	for (i = 0; i < m_size; i++)
 	{
@@ -563,6 +570,21 @@ void gcpCycleTool::OnRelease()
 			{
 				pAtom[i] = (gcpAtom*)m_pObject;
 			}
+			if (pAtom[i]) {
+				if (pMol == NULL) {
+					pMol = reinterpret_cast<gcpMolecule *> (pAtom[i]->GetMolecule ());
+					pMol->Lock (true);
+				}
+				// if group not in ModifiedObjects, add it and save it
+				pObject = pAtom[i]->GetGroup ();
+				Id = pObject->GetId ();
+				if (ModifiedObjects.find (Id) == ModifiedObjects.end ()) {
+					if (!pOp)
+						pOp = pDoc->GetNewOperation (GCP_MODIFY_OPERATION);
+					pOp->AddObject (pObject);
+					ModifiedObjects.insert (Id);
+				}
+			}
 		}
 		if (!pAtom[i])
 		{
@@ -585,6 +607,27 @@ void gcpCycleTool::OnRelease()
 		pBond = new gcpBond(pAtom[m_size - 1], pAtom[0], 1);
 		pDoc->AddBond(pBond);
 	}
+	pObject = pBond->GetGroup ();
+	if (pOp) {
+		ModifiedObjects.insert (pObject->GetId ());
+		set<string>::iterator it, end = ModifiedObjects.end();
+		for (it = ModifiedObjects.begin(); it != end; it++) {
+			pObject = pDoc->GetDescendant ((*it).c_str ());
+			if (pObject)
+				pOp->AddObject (pObject, 1);
+		}
+	} else {
+		pOp = pDoc->GetNewOperation (GCP_ADD_OPERATION);
+		pOp->AddObject (pObject);
+	}
+	pDoc->FinishOperation ();
+	if (pMol) {
+		pMol->Lock (false);
+puts("updating group");
+		pMol->EmitSignal (OnChangedSignal);
+puts("done updating group");
+	}
+	ModifiedObjects.clear ();
 }
 
 void gcpCycleTool::OnChangeState()
