@@ -4,9 +4,9 @@
  * GChemPaint
  * standaloneapp.cc
  *
- * Copyright (C) 2004
+ * Copyright (C) 2004-2005
  *
- * Developed by Jean Bréfort <jean.brefort@ac-dijon.fr>
+ * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -242,17 +242,17 @@ static GtkActionEntry entries[] = {
   { "FileMenu", NULL, N_("_File") },
 	  { "New", GTK_STOCK_NEW, N_("_New File"), NULL,
 		  N_("Create a new file"), G_CALLBACK (on_file_new) },
-	  { "Open", GTK_STOCK_OPEN, N_("_Open"), "<control>O",
+	  { "Open", GTK_STOCK_OPEN, N_("_Open..."), "<control>O",
 		  N_("Open a file"), G_CALLBACK (on_file_open) },
 	  { "Save", GTK_STOCK_SAVE, N_("_Save"), "<control>S",
 		  N_("Save the current file"), G_CALLBACK (on_file_save) },
-	  { "SaveAs", GTK_STOCK_SAVE_AS, N_("Save _As"), "<shift><control>S",
+	  { "SaveAs", GTK_STOCK_SAVE_AS, N_("Save _As..."), "<shift><control>S",
 		  N_("Save the current file with a different name"), G_CALLBACK (on_file_save_as) },
-	  { "SaveAsImage", GTK_STOCK_SAVE_AS, N_("Save As _Image"), "<control>I",
+	  { "SaveAsImage", GTK_STOCK_SAVE_AS, N_("Save As _Image..."), "<control>I",
 		  N_("Save the current file as an image"), G_CALLBACK (on_file_save_as_image) },
-	  { "Print", GTK_STOCK_PRINT, N_("_Print"), "<control>P",
+	  { "Print", GTK_STOCK_PRINT, N_("_Print..."), "<control>P",
 		  N_("Print the current file"), G_CALLBACK (on_print) },
-	  { "Properties", GTK_STOCK_PROPERTIES, N_("Prope_rties"), NULL,
+	  { "Properties", GTK_STOCK_PROPERTIES, N_("Prope_rties..."), NULL,
 		  N_("Modify the file's properties"), G_CALLBACK (on_properties) },
 	  { "Close", GTK_STOCK_CLOSE, N_("_Close"), "<control>W",
 		  N_("Close the current file"), G_CALLBACK (on_file_close) },
@@ -458,6 +458,8 @@ gcpStandaloneApp::gcpStandaloneApp(): gcpApplication()
 	SetMenu("Paste", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/EditMenu/Paste"));
 	SetMenu("Erase", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/EditMenu/Delete"));
 	SetMenu("Image", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/FileMenu/SaveAsImage"));
+	SetMenu("Save", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/FileMenu/Save"));
+	SetToolItem("Save", gtk_ui_manager_get_widget (m_UIManager, "/MainToolbar/Save"));
 	ActivateMenu("Undo", false);
 	ActivateMenu("Redo", false);
 	ActivateMenu("Copy", false);
